package de.hft.stuttgart.citydoctor2.gui.tree;

import de.hft.stuttgart.citydoctor2.gui.CheckStatus;
import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.scene.control.Button;
import javafx.scene.control.TreeCell;
import javafx.scene.paint.Color;

public class RenderableTreeCell extends TreeCell<Renderable> {
	
	private Renderable currentCheckable = null;
	private ChangeListener<CheckStatus> listener;
	
	@Override
	protected void updateItem(Renderable item, boolean empty) {
		super.updateItem(item, empty);
		if (!empty && item != null) {
			if (item instanceof ButtonRenderable) {
				Button b = new Button("More ...");
				b.setMaxWidth(Double.MAX_VALUE);
				setText(null);
				setGraphic(b);
				ButtonRenderable renderable = (ButtonRenderable) item;
				b.setOnAction(ae -> renderable.run());
				return;
			}
			updateColor(item.getStatus());
			setText(item.getText());
			setGraphic(null);
			if (currentCheckable != item) {
				if (listener != null) {
					// remove old Listener
					item.getStatusProperty().removeListener(listener);
				}
				// only add listener to the status property once
				listener = (obs, oldV, newV) -> Platform.runLater(() -> updateColor(newV));
				item.getStatusProperty().addListener(listener);
				currentCheckable = item;
			}
		} else {
			setTextFill(Color.BLACK);
			setText(null);
			setGraphic(null);
		}
	}

	private void updateColor(CheckStatus status) {
		if (status == CheckStatus.OK) {
			setTextFill(Color.GREEN);
		} else if (status == CheckStatus.ERROR) {
			setTextFill(Color.RED);
		} else {
			setTextFill(Color.BLACK);
		}
	}

}
